#include "simulat.h"
#include <math.h>
#include "nrutil.h"
#include "nr.h"
#include <stdio.h>

extern int NN,NL;

void MXW(float **mwmat,int *iup,int *idown)
{
	int i,j;
	
	for(i=1;i<=NL;i++)
	{
		mwmat[iup[i]][i]=-1.0;
		mwmat[idown[i]][i]=1.0;
	}
}

void WXP(float **wpmat,float *area, float *len,int *iup,int *idown)	
{
	int i,j;
	
	for(i=1;i<=NL;i++)
	{
		wpmat[i][iup[i]]=area[i]/len[i];
		wpmat[i][idown[i]]=-area[i]/len[i];
	}
}
			
void WXW(float **wwmat,float *k,float *w)
{
	int i,j;
	
	for(i=1;i<=NL;i++)
		wwmat[i][i]=-k[i]*fabs(w[i]);
}		

void HXW(float **hwmat,float *enth,float *mass,int *iup,int *idown,int *ifrom)
{
	int i,j;
	
	for(i=1;i<=NL;i++)
	{
		hwmat[iup[i]][i]=-enth[ifrom[i]]/mass[ifrom[i]];
		hwmat[idown[i]][i]=enth[ifrom[i]]/mass[ifrom[i]];
	}
}

void PXW(float **pwmat,float *c1,float *c2,float *enth,float *mass,int *iup, int *idown,int *ifrom)
{
	int i,j;
	
	for(i=1;i<=NL;i++)
	{
		pwmat[iup[i]][i]=-(c1[iup[i]]+c2[iup[i]]*enth[ifrom[i]]/mass[ifrom[i]]);
		pwmat[idown[i]][i]=c1[idown[i]]+c2[idown[i]]*enth[ifrom[i]]/mass[ifrom[i]];
	}	
}	
		
void HXW2(float **hwmat,float *enth,float *mass,float *w,float **mwmat,float dt,int *iup,int *idown,
		 int *ifrom)
{
	int i,j;
	float **hold;

	hold=matrix(1,NN,1,NN);
	
	for(i=1;i<=NL;i++)
	{
		hold[iup[i]][ifrom[i]]=hold[iup[i]][ifrom[i]]+w[i]*enth[ifrom[i]]*dt/(mass[ifrom[i]]*
								mass[ifrom[i]]);
		hold[idown[i]][ifrom[i]]=hold[idown[i]][ifrom[i]]-w[i]*enth[ifrom[i]]*dt/(mass[ifrom[i]]*
								mass[ifrom[i]]);
	}
	
	for(j=1;j<=NL;j++)
	{
		for(i=1;i<=NN;i++)
			hwmat[i][j]=hold[i][iup[j]]*mwmat[iup[j]][j]+hold[i][idown[j]]*mwmat[idown[j]][j];
	}	
	
	
	for(i=1;i<=NL;i++)
	{
		hwmat[iup[i]][i]=hwmat[iup[i]][i]-enth[ifrom[i]]/mass[ifrom[i]];
		hwmat[idown[i]][i]=hwmat[idown[i]][i]+enth[ifrom[i]]/mass[ifrom[i]];
	}

	free_matrix(hold,1,NN,1,NN);

}

void MTRANS(float **matran,float *mass,float *w,float dt,int *iup,int *idown,int *ifrom)
{
	int i,j,*indx;
	float **hold,**courant,*col,d;
	
	hold=matrix(1,NN,1,NN);
	col=vector(1,NN);
	indx=ivector(1,NN);
	courant=matrix(1,NN,1,NL);

	for(i=1;i<=NL;i++)
	{
		courant[iup[i]][i]=-w[i]*dt/mass[ifrom[i]];
		courant[idown[i]][i]=w[i]*dt/mass[ifrom[i]];
		
		hold[iup[i]][ifrom[i]]=hold[iup[i]][ifrom[i]]+courant[iup[i]][i];
		hold[idown[i]][ifrom[i]]=hold[idown[i]][ifrom[i]]+courant[idown[i]][i];
	}
	
	for(i=1;i<=NN;i++)
		hold[i][i]=1.0-hold[i][i];
		
	/* invert matran */		
		
	LUDCMP(hold,NN,indx,&d);
	
	for(j=1;j<=NN;j++)
	{
		for(i=1;i<=NN;i++) col[i]=0.0;
		col[j]=1.0;
		LUBKSB(hold,NN,indx,col);
		for(i=1;i<=NN;i++) matran[i][j]=col[i];
	}
	
	for(i=1;i<=NN;i++)
	{
		for(j=1;j<=NN;j++) matran[i][j]=hold[i][j];
	}

	free_matrix(courant,1,NN,1,NL);
	free_matrix(hold,1,NN,1,NN);
	free_vector(col,1,NN);
	free_ivector(indx,1,NN);
}

void PXW2(float **pwmat,float *c1,float *c2,float **matran,float **hwmat,int *iup,int *idown)	
{
	int i,j,k;
	
	for(j=1;j<=NL;j++)
	{
		for(i=1;i<=NN;i++)
		{
			pwmat[i][j]=0.0;
			for(k=1;k<=NN;k++)	pwmat[i][j]=pwmat[i][j]+matran[i][k]*hwmat[k][j]*c2[i];
		}
 	}
	
	for(i=1;i<=NL;i++)
	{
		pwmat[iup[i]][i]=pwmat[iup[i]][i]-c1[iup[i]];
		pwmat[idown[i]][i]=pwmat[idown[i]][i]+c1[idown[i]];
	}

}	

void MATMUL(float *a,float **b,float *c,int NR,int NC)
{
	int i,j;
	
	for(i=1;i<=NR;i++)
	{	
		a[i]=0.0;
		for(j=1;j<=NC;j++)	a[i]=a[i]+b[i][j]*c[j];			
	}
}		


void DxDT(float *dxdt,float *x,float **A,float *B,int NR,int NC)
{
	int i,j;
	
	for(i=1;i<=NR;i++)
	{	
		dxdt[i]=0.0;
		for(j=1;j<=NC;j++)	dxdt[i]=dxdt[i]+A[i][j]*x[j];			
		dxdt[i]=dxdt[i]+B[i];
	}
}												
